"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const IssueLocation_1 = require("./IssueLocation");
const normalizeSlash_1 = __importDefault(require("../utils/path/normalizeSlash"));
class IssueWebpackError extends Error {
    constructor(message, context, issue) {
        super(message);
        this.issue = issue;
        this.hideStack = true;
        // to display issue location using `loc` property, webpack requires `error.module` which
        // should be a NormalModule instance.
        // to avoid such a dependency, we do a workaround - error.file will contain formatted location instead
        if (issue.file) {
            const parts = [normalizeSlash_1.default(path_1.relative(context, issue.file))];
            if (issue.location) {
                parts.push(IssueLocation_1.formatIssueLocation(issue.location));
            }
            this.file = parts.join(' ');
        }
        Error.captureStackTrace(this, this.constructor);
    }
}
exports.IssueWebpackError = IssueWebpackError;
