"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const ts = __importStar(require("typescript"));
const TypeScriptIssueFactory_1 = require("../issue/TypeScriptIssueFactory");
const ControlledWatchCompilerHost_1 = require("./ControlledWatchCompilerHost");
const TypeScriptVueExtension_1 = require("../extension/vue/TypeScriptVueExtension");
const ControlledWatchSolutionBuilderHost_1 = require("./ControlledWatchSolutionBuilderHost");
const ControlledTypeScriptSystem_1 = require("./ControlledTypeScriptSystem");
function createTypeScriptReporter(configuration) {
    const extensions = [];
    let system;
    let watchCompilerHost;
    let watchSolutionBuilderHost;
    let watchProgram;
    let solutionBuilder;
    const diagnosticsPerProject = new Map();
    if (configuration.extensions.vue.enabled) {
        extensions.push(TypeScriptVueExtension_1.createTypeScriptVueExtension(configuration.extensions.vue));
    }
    function getProjectNameOfBuilderProgram(builderProgram) {
        return builderProgram.getProgram().getCompilerOptions().configFilePath;
    }
    function getDiagnosticsOfBuilderProgram(builderProgram) {
        const diagnostics = [];
        if (typeof builderProgram.getConfigFileParsingDiagnostics === 'function') {
            diagnostics.push(...builderProgram.getConfigFileParsingDiagnostics());
        }
        if (typeof builderProgram.getOptionsDiagnostics === 'function') {
            diagnostics.push(...builderProgram.getOptionsDiagnostics());
        }
        if (configuration.diagnosticOptions.syntactic) {
            diagnostics.push(...builderProgram.getSyntacticDiagnostics());
        }
        if (configuration.diagnosticOptions.semantic) {
            diagnostics.push(...builderProgram.getSemanticDiagnostics());
        }
        if (configuration.diagnosticOptions.declaration) {
            diagnostics.push(...builderProgram.getDeclarationDiagnostics());
        }
        if (configuration.diagnosticOptions.global) {
            diagnostics.push(...builderProgram.getGlobalDiagnostics());
        }
        return diagnostics;
    }
    return {
        getReport: ({ changedFiles = [], deletedFiles = [] }) => __awaiter(this, void 0, void 0, function* () {
            if (!system) {
                system = ControlledTypeScriptSystem_1.createControlledTypeScriptSystem();
            }
            // clear cache to be ready for next iteration and to free memory
            system.clearCache();
            if (configuration.build) {
                // solution builder case
                // ensure watch solution builder host exists
                if (!watchSolutionBuilderHost) {
                    watchSolutionBuilderHost = ControlledWatchSolutionBuilderHost_1.createControlledWatchSolutionBuilderHost(configuration.tsconfig, configuration.compilerOptions, // assume that these are valid ts.CompilerOptions
                    system, ts.createSemanticDiagnosticsBuilderProgram, undefined, undefined, undefined, undefined, (builderProgram) => {
                        const projectName = getProjectNameOfBuilderProgram(builderProgram);
                        const diagnostics = getDiagnosticsOfBuilderProgram(builderProgram);
                        // update diagnostics
                        diagnosticsPerProject.set(projectName, diagnostics);
                    }, extensions);
                    solutionBuilder = undefined;
                }
                // ensure solution builder exists
                if (!solutionBuilder) {
                    solutionBuilder = ts.createSolutionBuilderWithWatch(watchSolutionBuilderHost, [configuration.tsconfig], {});
                    solutionBuilder.build();
                }
            }
            else {
                // watch compiler case
                // ensure watch compiler host exists
                if (!watchCompilerHost) {
                    watchCompilerHost = ControlledWatchCompilerHost_1.createControlledWatchCompilerHost(configuration.tsconfig, configuration.compilerOptions, // assume that these are valid ts.CompilerOptions
                    system, ts.createSemanticDiagnosticsBuilderProgram, undefined, undefined, (builderProgram) => {
                        const projectName = getProjectNameOfBuilderProgram(builderProgram);
                        const diagnostics = getDiagnosticsOfBuilderProgram(builderProgram);
                        // update diagnostics
                        diagnosticsPerProject.set(projectName, diagnostics);
                    }, extensions);
                    watchProgram = undefined;
                }
                // ensure watch program exists
                if (!watchProgram) {
                    watchProgram = ts.createWatchProgram(watchCompilerHost);
                }
            }
            changedFiles.forEach((changedFile) => {
                if (system) {
                    system.invokeFileChanged(changedFile);
                }
            });
            deletedFiles.forEach((removedFile) => {
                if (system) {
                    system.invokeFileDeleted(removedFile);
                }
            });
            // wait for all queued events to be processed
            yield system.waitForQueued();
            // aggregate all diagnostics and map them to issues
            const diagnostics = [];
            diagnosticsPerProject.forEach((projectDiagnostics) => {
                diagnostics.push(...projectDiagnostics);
            });
            let issues = TypeScriptIssueFactory_1.createIssuesFromTsDiagnostics(diagnostics);
            extensions.forEach((extension) => {
                if (extension.extendIssues) {
                    issues = extension.extendIssues(issues);
                }
            });
            return issues;
        }),
    };
}
exports.createTypeScriptReporter = createTypeScriptReporter;
